<?php

namespace Fastrack\Core\Model;

use Fastrack\Core\Util\ModelUtilities;
use SilverStripe\Assets\File;
use SilverStripe\Control\Controller;
use SilverStripe\Control\Director;
use SilverStripe\Forms\FieldList;
use SilverStripe\ORM\DataObject;
use SilverStripe\ORM\ManyManyList;
use SilverStripe\Security\Permission;
use SilverStripe\Security\PermissionProvider;
use SilverStripe\Versioned\Versioned;
use TractorCow\Fluent\Extension\FluentVersionedExtension;

/**
 * @property string Title
 * @property string Description
 * @property string SvgCode
 * @property string FontAwesomeClasses
 * @property string Key
 * @property int ImageID
 * @property int SharedFeatureID
 * @property int FeatureGroupID
 * @method Imgix Image()
 * @method File Svg()
 * @method SharedFeature SharedFeature()
 * @method ManyManyList FeatureGroups()
 * @method ManyManyList BookDirectFeatures()
 * @extends Versioned
 */
class Feature extends DataObject implements PermissionProvider
{

	const PERMISSIONS_KEY = 'MANAGE_FEATURE';

	private static string $table_name = 'Feature';

	private static array $extensions = [
		Versioned::class,
		FluentVersionedExtension::class,
	];

	private static array $db = [
		'Title' => 'Varchar',
		'Description' => 'Text',
		'Sort' => 'Int',
	];

	private static array $has_one = [
		'Image' => Imgix::class,
		'FeatureGroup' => FeatureGroup::class,
		'SharedFeature' => SharedFeature::class,
	];

	private static array $owns = [
		'Image',
	];

	private static array $translate = [
		'Title',
		'Description',
	];

	private static array $hidden_fields = [
		'Sort',
		'FeatureGroupID',
		'SharedFeatureID',
	];

	private static string $default_sort = 'Sort';

	public function getCMSFields(): FieldList
	{
		$fields = parent::getCMSFields();
		ModelUtilities::hideFields($this, $fields);
		if ($this->SharedFeature()->exists()) {
			$info = '<i class="font-icon-info-circled"></i> This is a shared feature. ' .
					'edit this from [Properties] > [Features]';
			$title = $fields->dataFieldByName('Title');
			$title?->setDisabled(true);
			$title?->setDescription($info);
			$fields->dataFieldByName('Description')?->setDisabled(true);
			$fields->dataFieldByName('Image')?->setDisabled(true);
		}
		return $fields;
	}

	public function getTitle(): string
	{
		if ($this->SharedFeature()->exists()) {
			return $this->SharedFeature()->Title ?? '';
		}
		return $this->getField('Title') ?? '';
	}

	public function getDescription(): string
	{
		if ($this->SharedFeature()->exists()) {
			return $this->SharedFeature()->Description ?? '';
		}
		return $this->getField('Description') ?? '';
	}

	public function getImage(): Imgix
	{
		if ($this->SharedFeature()->exists()) {
			return $this->SharedFeature()->Image();
		}
		return $this->Image();
	}

	public function providePermissions(): array
	{
		$permissions = [
			self::PERMISSIONS_KEY => [
				'name' => 'Manage features',
				'category' => 'Features',
			],
		];
		$this->extend('updatePermissions', $permissions);
		return $permissions;
	}

	public function canView($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canEdit($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canDelete($member = null): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

	public function canCreate($member = null, $context = array()): bool|int
	{
		return Permission::check([self::PERMISSIONS_KEY]);
	}

}
